/* 
   -----------------------------------------------------------------------
   Program.....: client.cc
   Programmer..: Bernd Hohenester  (newsletter@hobe.de)
   Date........: January 16, 2004 
   Notes.......: A custom class for automated survey of users, who 
                 work with the application at the same time.
                 Manages exclusive access and restricts other 
                 users from logging in.

   Written for.: dBase Plus 2.01
   Rev. History: 16.01.2004   0.1 - Original version

   Calls.......: all encapsulated in this class.
   Called by...: Any

   ---------------------------------------------------------------------

   This custom object uses four functions:
   
   - LogIn()     - checks if table sys_client.dbf exists
                 - creates the table if not
                 - checks for exclusive mode of another user
                 - logs in the current user
                 
                 Usage:
                 
                 In the startup-routine of the application
                 
                 set procedure to client.cc additive
                 _app.cl = new client()
                 if not _app.cl.login("<yourDatabaseName>")
                   quit
                 endif
                 
                 Keep the client object open for later use!
                 
                 
   - Exclusive() - checks for other users logged in
                 - optional informs the users with network message 
                   service
                 - logs in exclusive mode
                 
                 Usage:
                 
                 In the routine, i.e. for packing the tables
                 
                 if not _app.cl.exclusive()
                   return
                 endif
                 
                 
   - Unlock()    - only leaves exclusive mode
    
                 Usage:
                 
                 At the end of the packing-routine
                 
                 _app.cl.unlock()
                 
                 
   - LogOut()    - logs the current user out
                 - deletes table sys_client.dbf when no one else is 
                   logged in
                 
                 Usage:
                 
                 In the shutdown-routine of the application
                 
                 _app.cl.logout()
                 release object _app.cl
                 _app.cl = null
*/

class client(ParentObj) of object(ParentObj) custom
  
   // custom properties
   this.DatabaseName = ""
   this.id = 0
   this.HostName = class::GetSystemName()

   function login(DatabaseName)

      // does table sys_client.dbf exist?

      local d, q, lRueck

      private cDatabaseName
      cDatabaseName = DatabaseName
      this.DatabaseName = cDatabaseName

      // first open database
      d = new database()
      with (d)
         databasename := this.DatabaseName
         active       := true
      endwith


      if not d.tableExists( "sys_client.dbf" )
         create table ":&cDatabaseName.:sys_client.dbf" ( ;
                      Id       AUTOINC,;
                      LastUser CHAR(254),;
                      LastHost CHAR(254),;
                      LastDate DATE,;
                      LastTime CHAR(5),;
                      LastExcl BOOLEAN )
      endif


      // open table
      q = new query()
      with (q)
         database := d
         sql      := [select * from "sys_client.dbf"]
         active   := true
      endwith


      // check for exclusive mode
      lRueck = true
      do
         if q.rowset.fields["LastExcl"].value == true
            // can't log in
            cMsg = "The program is currently locked for maintenance purposes." +;
                   chr(13) + "User: " + q.rowset.fields[ "LastUser" ].value.righttrim() +;
                   " at " + q.rowset.fields["LastHost"].value.righttrim() + chr(13) + ;
                   "Please try again later."

            msgbox( cMsg, "Exclusive Access", 0 + 48 )
    
            lRueck = false
            exit
         endif
      until not q.rowset.next()


      if lRueck
         // login the current user
         q.rowset.beginappend()
         q.rowset.fields["LastUser"].value = id()
         q.rowset.fields["LastHost"].value = this.HostName
         q.rowset.fields["LastDate"].value = date()
         q.rowset.fields["LastTime"].value = time()
         q.rowset.fields["LastExcl"].value = false
         q.rowset.save()
         this.id = q.rowset.fields["Id"].value
      endif
  
  
      q.active := false
      release object q
      q = null
  
      d.active := false
      release object d
      d = null

   return(lRueck)

   function logout

      local d, q, nCount
  
      d = new database()
      with (d)
         databasename := this.DatabaseName
         active       := true
      endwith

      q = new query()
      with (q)
         database := d
         sql      := [select * from "sys_client.dbf"]
         active   := true
      endwith

      q.rowset.first()
      do
         if q.rowset.fields["Id"].value == this.id
            q.rowset.delete()
            exit
         endif
      until not q.rowset.next()
  

      nCount = q.rowset.count()
  
      q.active := false
      release object q
      q = null
    
      if nCount == 0
         // noone else is logged in - delete table
         d.dropTable("sys_client.dbf")
      endif

      d.active := false
      release object d
      d = null

   return

   function exclusive
  
      local lRueck, d, q, cMsg
      lRueck = true

      d = new database()
      with (d)
         databasename := this.DatabaseName
         active := true
      endwith

      q = new query()
      with (q)
         database := d
         sql      := [select * from "sys_client.dbf"]
         active   := true
      endwith

      if q.rowset.count() > 1
         lRueck = false
         cMsg = "The following users are logged in:" + chr(13)
    
         q.rowset.first()
         do
            // do not show own login
            if q.rowset.fields["Id"].value <> this.id
               cMsg += q.rowset.fields[ "LastUser" ].value.righttrim() +;
                       " at " + ;
                       q.rowset.fields["LastHost"].value.righttrim() +;
                       chr(13)
            endif
         until not q.rowset.next()
         cMsg += "Do you want to inform the users for logout?"

         if msgbox( cMsg, "No exclusive access", 4 + 48 ) == 6
            q.rowset.first()
            do
               if q.rowset.fields["Id"].value <> this.id
                  cMsg = "Please log out for maintenance purposes!"
                  run(true, [net send /domain:] + ;
                      q.rowset.fields["LastHost"].value.righttrim() +;
                      [ "] + cMsg + ["])
               endif
            until not q.rowset.next()
         endif
      endif  
  

      if lRueck
         q.rowset.first()
         do
            if q.rowset.fields["Id"].value == this.id
               exit
            endif
         until not q.rowset.next()
  
         q.rowset.beginedit()
         q.rowset.fields["LastExcl"].value = true
         q.rowset.save()
      endif
  
      q.active := false
      release object q
      q = null
  
      d.active := false
      release object d
      d = null

   return (lRueck)

   function unlock

      local d, q
  
      d = new database()
      with (d)
         databasename := this.DatabaseName
         active       := true
      endwith

      q = new query()
      with (q)
         database := d
         sql      := [select * from "sys_client.dbf"]
         active   := true
      endwith

      q.rowset.first()
      do
         if q.rowset.fields["Id"].value == this.id
            exit
         endif
      until not q.rowset.next()
  
      q.rowset.beginedit()
      q.rowset.fields["LastExcl"].value = false
      q.rowset.save()
  
      q.active := false
      release object q
      q = null
  
      d.active := false
      release object d
      d = null

   return (true)


   function GetSystemName
      /*
         ---------------------------------------------------------------
         Posted by Jim Sare in the Visual dBASE Newsgroups
         February 11, 1999

         This uses api calls to obtain the "computer name".
         This was posted in response to a suggestion to obtain
         this information from the Registry, and Jim pointed
         out that the next release of the OS (Win 2000, or 
         whatever) could change the registry key ...

         Usage:
            ? GetSystemName()
         ---------------------------------------------------------------
      */
      local xBuff, nLen

      if type( "GetComputerName" ) # "FP"
         extern CLOGICAL GetComputerName( CSTRING, CPTR CULONG ) kernel32;
                from "GetComputerNameA"
      endif

      #define MAX_COMPUTERNAME_LENGTH 15

      nLen = MAX_COMPUTERNAME_LENGTH + 1

      xBuff = replicate( chr(0), nLen )
      if GetComputerName( xBuff, nLen )
         RETURN left( xBuff.string, nLen )
      endif

   RETURN ""
   // eof: GetSystemName()


endclass

// End of file: client.cc
